/**

   slave control panel for TXN



   MIT License
   Copyright (C) 2019 THomas Stephens
*/

#include <Wire.h>
#include <arduino.h>

#define SLAVE_ADDRESS 50 // change this for each slave instance, Minimum number is 8, Must match entry in master table of slave IDs.
#define MAX_WIRE_LENGTH 2 // max message size to expect 
#define STATEBASE 65  // convert integer route number to ASCII range data for I2C.

#define DEBUG_PIN     13      // for debugging
#define DEBUG_ON  HIGH
#define DEBUG_OFF  LOW
#define ROUTECOUNT 4  // numper of options for user input



////Pin connected to DS of 74HC595
int dataPin = A0; //                    Blue
//Pin connected to ST_CP of 74HC595
int latchPin = A1; //                    green
//Pin connected to SH_CP of 74HC595
int clockPin = A2; //                   yellow



// array values for button scanning.
int button_pins[ROUTECOUNT] = {4, 5, 6, 7}; // index 1 to 8, 0 not used.


// anything modified by the ISR must be volatile!
volatile byte      receivedData;      // sent by master, used to set LED display
volatile byte      rawdata[MAX_WIRE_LENGTH + 1];
volatile boolean   ReadByMaster;
volatile int   button_value;

byte current_value;
byte new_value;




/**
    callback from Wire - the master is writing to us, I2C conversation is in progress...
    Possibilities:
    1.  The master is requesting current value from node
    2.  The master is updating status with data from another node
*/

// ------------------- reveive event ---------------------------
void receiveEvent(int bytesReceived) {
  for (int x = 0; x < bytesReceived; x++) {
    if (x < MAX_WIRE_LENGTH) {
      rawdata[x] = Wire.read();
    } else {
      (void)Wire.read();    // throw extras away
    }
  }
  receivedData = rawdata[0] - STATEBASE;
  Serial.print("  ***read from master ");
  Serial.println(receivedData);

}

// ---------requestevent ----------------------
void requestEvent(void) {

  volatile byte returndata[2];
  returndata[0] = button_value + STATEBASE;
  returndata[1] = 0;

  Wire.write(returndata[0]);              // sends one byte

  Serial.print("  button value request from master ");
  Serial.println(button_value);
  button_value = 16; // mark as sent
}

// ------------------- register write
// This method sends bits to the shift register:

void registerWrite(int whichBit, int whichState = HIGH) {
  // the bits you want to send
  byte bitsToSend = 0;

  // turn off the output so the pins don't light up
  // while you're shifting bits:
  digitalWrite(latchPin, LOW);

  // turn on one bit in bitsToSend:
  bitWrite(bitsToSend, whichBit, whichState);

  // shift the bits out:
  shiftOut(dataPin, clockPin, MSBFIRST, bitsToSend);

  // turn on the output so the LEDs can light up:
  digitalWrite(latchPin, HIGH);

}

// ----------------------------setup -------------------
void setup() {
  //Initialize serial and wait for port to open:
  Serial.begin(9600);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }
  Serial.println("I2C Slave - Thomas Stephens");
  Serial.println("4 position control panel with button to set each LED on, and read/write from I2C master");
  Serial.print("Address: ");
  Serial.println(SLAVE_ADDRESS);

  //set pins to output because they are used in the shift register code
  pinMode(latchPin, OUTPUT);
  pinMode(clockPin, OUTPUT);
  pinMode(dataPin, OUTPUT);

  // set each button pin for input, default HIGH.
  pinMode(DEBUG_PIN, OUTPUT);
  for (int x = 0; x < ROUTECOUNT; x++) {
    pinMode(button_pins[x], INPUT_PULLUP);

  }
  // I2C pin initializations handled Wire subsystem

  // set defaults
  Wire.begin(SLAVE_ADDRESS);
  Wire.onRequest(requestEvent);
  Wire.onReceive(receiveEvent);

  new_value = 0; // default
  current_value = 0; // force first update.
  receivedData = 0;
  registerWrite(current_value, HIGH);

  // debug
  digitalWrite(DEBUG_PIN, DEBUG_ON);  // light up the pin, since someone is talking to us...

} // end startup
// ------------------loop ------------------
void loop() {
  Serial.print("  receivedData ");
  Serial.print(receivedData);
  Serial.print(" current value ");
  Serial.print(current_value);
  Serial.print("  button value ");
  Serial.println(button_value);

  // check for data from master
  // note this is set by interrupt routine, this runs outside the main loop when master sends data
  if (receivedData != current_value) {
    current_value = receivedData; // got new number from Master
  }
  // do check for local button push

  // find first LOW button
  for (uint8_t x = 0; x < ROUTECOUNT; x++) {
    int state;
    state = digitalRead(button_pins[x]);
    Serial.print(" x= ");
    Serial.print(x);
    if (state == LOW) {
      delay(10);
      if (digitalRead(button_pins[x]) == LOW)
      {
        button_value = x;
        Serial.print("  button pushed ");
        Serial.println(button_value);
        break;
      }

    }
    Serial.println(".");
  }

  // if new value not same as old, send update bit pattern to shift out register chip
  if (new_value != current_value) {
    // set LED group based on value.
    new_value = current_value;
// for non-linear states, I.E. more than one LED on, create a s call SetLed(new_value) function to determine bit patters for output shift register.


    Serial.print("new value set: ");
    Serial.println(new_value);
    registerWrite(current_value, HIGH); // update LED array. Note that Bit number for shift register is 0 based. By setting LOW, turning off all LED indicators.
  }
  //Serial.print("ReadByMaster ");
  //Serial.print(ReadByMaster);

  delay(1000); // cycle each 1/2 second
  // debug code

} // end main loop
//--------------------------------- SetLed function code
void SetLed(int state)
{
  
}
